/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConfiguratorFrame.java 20821 2011-02-02 16:59:06Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.configurationframe;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionListener;

import org.ow2.jonas.autostart.configuration.Configuration;
import org.ow2.jonas.autostart.configuration.JDBCDatasource;
import org.ow2.jonas.autostart.configuration.Protocol;
import org.ow2.jonas.autostart.configuration.Service;
import org.ow2.jonas.autostart.utility.SpringUtilities;

/**
 * This class creates main panel with ConfiguratorFrame fields.
 * @author skabore
 */
public class ConfiguratorFrame extends JPanel {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = -3756410766870650588L;

    /**
     * The content panel.
     */
    private JPanel contentPanel;

    /**
     * The icon label.
     */
    private JLabel iconLabel;

    /**
     * The separator.
     */
    private JSeparator separator;

    /**
     * The text label.
     */
    private JLabel textLabel;

    /**
     * The title panel.
     */
    private JPanel titlePanel;

    /**
     * The validate button.
     */
    private JButton jbValidateConfig;

    /**
     * The cancel button.
     */
    private JButton jbCancelConfig;

    /**
     * The browseJdbc button.
     */
    private JButton jbBrowseJdbc;

    /**
     * The selectAll button.
     */
    private JButton jbSelectAll;

    /**
     * The deselectAll button.
     */
    private JButton jbDeselectAll;

    /**
     * The jbSaveDatasource button.
     */
    private JButton jbSaveDatasource;

    /**
     * The jbNewDatasource button.
     */
    private JButton jbNewDatasource;

    /**
     * The jbDeleteDatasource button.
     */
    private JButton jbDeleteDatasource;

    /**
     * The jbResetDatasource button.
     */
    private JButton jbResetDatasource;

    /**
     * The jbAdvancedDsConf button.
     */
    private JButton jbAdvancedDsConf;

    /**
     * The jbSaveAdvancedDsConf button.
     */
    private JButton jbSaveAdvancedDsConf;

    /**
     * The jbResetAdvancedDsConf button.
     */
    private JButton jbResetAdvancedDsConf;

    /**
     * The label.
     */
    private JLabel label;

    /**
     * The jndiName textfield.
     */
    private JTextField jndiName;

    /**
     * The datasourceName textfield.
     */
    private JTextField datasourceName;

    /**
     * The jdbcDriverPath textfield.
     */
    private JTextField jdbcDriverPath;

    /**
     * The jdbcDriverUrl textfield.
     */
    private JTextField jdbcDriverUrl;

    /**
     * The jdbcDriverUrl textfield.
     */
    private JTextField driverName;

    /**
     * The datasourceClassName textfield.
     */
    private JTextField datasourceClassName;

    /**
     * The database datasourceUrl textfield.
     */
    private JTextField datasourceUrl;

    /**
     * The database datasourceMapper textfield.
     */
    private JTextField datasourceMapper;

    /**
     * The database raName textfield.
     */
    private JTextField raName;

    /**
     * The database rarLink textfield.
     */
    private JComboBox rarLink;

    /**
     * The user textfield.
     */
    private JTextField user;

    /**
     * The maxAge textfield.
     */
    private JTextField maxAge;

    /**
     * The maxOpenTime textfield.
     */
    private JTextField maxOpenTime;

    /**
     * The minPool textfield.
     */
    private JTextField minPool;

    /**
     * The maxPool textfield.
     */
    private JTextField maxPool;

    /**
     * The maxWaitTime textfield.
     */
    private JTextField maxWaitTime;

    /**
     * The maxWaiters textfield.
     */
    private JTextField maxWaiters;

    /**
     * The pstmtPoolMax textfield.
     */
    private JTextField pstmtPoolMax;

    /**
     * The sampling textfield.
     */
    private JTextField sampling;

    /**
     * The adjustPeriod textfield.
     */
    private JTextField adjustPeriod;

    /**
     * The test textfield.
     */
    private JTextField test;

    /**
     * The user password field.
     */
    private JPasswordField passwd;

    /**
     * The dbms combobox.
     */
    private JComboBox dbms;

    /**
     * The datasource items.
     */
    private List<String> datasourceItems;

    /**
     * The driverVersion combobox.
     */
    private JComboBox driverVersionComboBox;

    /**
     * The datasourceVersion textfield.
     */
    private JTextField driverVersionTextField;

    /**
     * The checkingLevel combobox.
     */
    private JComboBox checkingLevel;

    /**
     * The copyJdbc radiobutton.
     */
    private JRadioButton copyJdbc;

    /**
     * The downloadJdbcFromUrl radiobutton.
     */
    private JRadioButton downloadJdbcFromUrl;

    /**
     * The downloadJdbcFromVersion radiobutton.
     */
    private JRadioButton downloadJdbcFromVersion;

    /**
     * The tabbedPane.
     */
    private JTabbedPane tabbedPane;

    /**
     * The httpPort textfield.
     */
    private JTextField httpPort;

    /**
     * The httpsPort textfield.
     */
    private JTextField httpsPort;

    /**
     * The hsqlPort textfield.
     */
    private JTextField hsqlPort;

    /**
     * The carolPort textfield.
     */
    private JTextField carolPort;

    /**
     * The carolLabel label.
     */
    private JLabel carolLabel;

    /**
     * The jrmpLabel label.
     */
    private JLabel jrmpLabel;

    /**
     * The irmiLabel label.
     */
    private JLabel irmiLabel;

    /**
     * The iiopLabel label.
     */
    private JLabel iiopLabel;

    /**
     * The lmiLabel label.
     */
    private JLabel lmiLabel;

    /**
     * The redirectPort textfield.
     */
    private JTextField redirectPort;

    /**
     * The implemClass textfield.
     */
    private JTextField implemClass;

    /**
     * The jrmpLabel textfield.
     */
    private JTextField jrmp;

    /**
     * The irmiLabel textfield.
     */
    private JTextField irmi;

    /**
     * The lmi textfield.
     */
    private JTextField lmi;

    /**
     * The iiopLabel textfield.
     */
    private JTextField iiop;

    /**
     * The jrmp RadioButton.
     */
    private JCheckBox cbJrmp;

    /**
     * The iiop RadioButton.
     */
    private JCheckBox cbIiop;

    /**
     * The irmi RadioButton.
     */
    private JCheckBox cbIrmi;

    /**
     * The lmi RadioButton.
     */
    private JCheckBox cbLmi;

    /**
     * The useSsl checkBox.
     */
    private JCheckBox useSsl;

    /**
     * The jvmRoute checkBox.
     */
    private JCheckBox jvmRoute;

    /**
     * The jkPort checkBox.
     */
    // private JCheckBox jkPort;

    /**
     * The mainPanel.
     */
    private JPanel mainPanel;

    /**
     * The webContainerPanel.
     */
    private JPanel webContainerPanel;

    /**
     * The protocolPanel.
     */
    private JPanel protocolPanel;

    /**
     * The servicesPanel.
     */
    private JPanel servicesPanel;

    /**
     * The datasourcePanel.
     */
    private JPanel datasourcePanel;

    /**
     * The advancedConfigPanel.
     */
    private JPanel advancedConfigPanel;

    /**
     * The protocol Components panel.
     */
    private JPanel protocolPanelComponents;

    /**
     * The HSQL Components panel.
     */
    private JPanel hsqlPanelComponents;

    /**
     * The JMS Components panel.
     */
    private JPanel jmsPanelComponents;

    /**
     * The lines number of protocolPanelComponents.
     */
    private int lines = 0;

    /**
     * The columns number of protocolPanelComponents.
     */
    private int columns = 2;

    /**
     * The buttonPanel.
     */
    private ArrayList<JCheckBox> servicesCheckBox;

    /**
     * The jdbcDatasources.
     */
    private List<String> jdbcDatasources;

    /**
     * The datasources list.
     */
    private JList datasourceList;

    /**
     * The datasources list.
     */
    private JList advancedDatasourceList;

    /**
     * The hsql db name.
     */
    private JTextField dbName;

    /**
     * The hsql db port.
     */
    private JTextField dbPort;

    /**
     * The hsql db user name.
     */
    private JTextField dbUser;

    /**
     * The hsql db user password.
     */
    private JPasswordField dbUserPasswd;

    /**
     * The jms port number.
     */
    private JTextField jmsPort;

    /**
     * The jms queues.
     */
    private JTextField jmsQueues;

    /**
     * The jms topics.
     */
    private JTextField jmsTopics;

    /**
     * The jms queues list.
     */
    private JList jmsQueuesList;

    /**
     * The jms topics list.
     */
    private JList jmsTopicsList;

    /**
     * The jms queues items.
     */
    private List<String> jmsQueuesItems;

    /**
     * The jms topics items.
     */
    private List<String> jmsTopicsItems;

    /**
     * The jms queues add button.
     */
    private JButton jbAddJmsQueues;

    /**
     * The jms topics add button.
     */
    private JButton jbAddJmsTopics;

    /**
     * The jms queues remove button.
     */
    private JButton jbRemoveJmsQueues;

    /**
     * The jms topics remove button.
     */
    private JButton jbRemoveJmsTopics;

    /**
     * The applyConfiguration checkBox.
     */
    // private JCheckBox applyConfiguration;

    /**
     * The configuration.
     */
    private Configuration configuration;

    /**
     * The constructor.
     * @param configuration the configuration librairy
     * @param stepId, the step identifier
     * @param steps, the required steps to create a starter
     */
    public ConfiguratorFrame(final Configuration configuration, final int stepId, final int steps) {
        super();
        this.configuration = configuration;
        this.datasourceItems = new ArrayList<String>();
        // datasourceVersionItems = new Vector <String> ();
        this.contentPanel = getContentPanel();
        this.contentPanel.setBorder(new EmptyBorder(new Insets(10, 10, 10, 10)));

        ImageIcon icon = getImageIcon();

        this.titlePanel = new JPanel();
        this.textLabel = new JLabel();
        this.iconLabel = new JLabel();
        this.separator = new JSeparator();

        setLayout(new BorderLayout());

        this.titlePanel.setLayout(new java.awt.BorderLayout());
        this.titlePanel.setBackground(Color.gray);

        this.textLabel.setBackground(Color.gray);
        this.textLabel.setFont(new Font("MS Sans Serif", Font.BOLD, 14));
        this.textLabel.setText("JOnAS server configuration.");
        this.textLabel.setBorder(new EmptyBorder(new Insets(10, 10, 10, 10)));
        this.textLabel.setOpaque(true);

        this.iconLabel.setBackground(Color.gray);
        if (icon != null) {
            this.iconLabel.setIcon(icon);
        }

        this.titlePanel.add(this.textLabel, BorderLayout.CENTER);
        this.titlePanel.add(new JLabel(stepId + "/" + steps + "  "), BorderLayout.EAST);
        this.titlePanel.add(this.separator, BorderLayout.SOUTH);

        add(this.titlePanel, BorderLayout.NORTH);
        JPanel secondaryPanel = new JPanel(new BorderLayout());
        secondaryPanel.add(this.contentPanel, BorderLayout.NORTH);
        add(secondaryPanel, BorderLayout.WEST);
    }

    /**
     * @return configuration.
     */
    public Configuration getConfiguration() {
        return this.configuration;
    }

    /**
     * @return contentPanel1 the content panel.
     */
    private JPanel getContentPanel() {

        JPanel contentPanel1 = new JPanel(new BorderLayout());

        this.tabbedPane = new JTabbedPane();

        this.mainPanel = new JPanel(new SpringLayout());
        this.webContainerPanel = new JPanel(new BorderLayout());
        this.protocolPanel = new JPanel(new BorderLayout());
        this.servicesPanel = new JPanel(new BorderLayout());
        this.datasourcePanel = new JPanel(new BorderLayout());
        this.advancedConfigPanel = new JPanel(new BorderLayout());
        this.jmsPanelComponents = new JPanel(new BorderLayout());
        this.hsqlPanelComponents = new JPanel(new BorderLayout());

        List<String> ds = new ArrayList<String>();
        ds.add("");
        ds.add("PostgreSQL");
        ds.add("Oracle");
        ds.add("MySQL");
        ds.add("FirebirdSQL");
        ds.add("InterBase");
        ds.add("Sybase");
        ds.add("InstantDB");
        ds.add("McKoi");
        ds.add("DB2");
        ds.add("HSQL");
        Collections.sort(ds);
        this.dbms = new JComboBox(new Vector<String>(ds));
        this.driverVersionComboBox = new JComboBox();
        this.driverVersionComboBox.setPreferredSize(new Dimension(80, 0));
        this.checkingLevel = new JComboBox();
        this.copyJdbc = new JRadioButton("Copy driver from disk");
        this.downloadJdbcFromVersion = new JRadioButton("Get driver from this version");
        this.downloadJdbcFromUrl = new JRadioButton("Get driver from this URL");
        ButtonGroup groupCopy = new ButtonGroup();
        groupCopy.add(this.downloadJdbcFromUrl);
        groupCopy.add(this.downloadJdbcFromVersion);
        groupCopy.add(this.copyJdbc);
        this.copyJdbc.setSelected(true);

        this.cbJrmp = new JCheckBox("JRMP");
        this.cbIiop = new JCheckBox("IIOP");
        this.cbIrmi = new JCheckBox("IRMI");
        this.cbLmi = new JCheckBox("LMI");
        selectProtocolsCheckBox();

        this.httpPort = new JTextField(20);
        this.redirectPort = new JTextField(20);
        this.irmi = new JTextField(20);
        this.lmi = new JTextField(20);
        this.jrmp = new JTextField(30);
        this.iiop = new JTextField(20);
        this.jdbcDriverPath = new JTextField(20);
        this.jdbcDriverUrl = new JTextField(20);
        this.driverVersionTextField = new JTextField(20);
        this.driverName = new JTextField(20);
        this.jndiName = new JTextField(20);
        this.datasourceName = new JTextField(20);
        this.datasourceUrl = new JTextField(20);
        this.datasourceClassName = new JTextField(20);
        this.datasourceMapper = new JTextField(20);
        this.raName = new JTextField(20);
        this.rarLink = new JComboBox(new String[] {"", "JOnASJDBC_DM", "JOnASJDBC_DS", "JOnASJDBC_CP", "JOnASJDBC_XA"});
        this.user = new JTextField(20);
        this.passwd = new JPasswordField(20);
        this.maxAge = new JTextField(20);
        this.maxOpenTime = new JTextField(20);
        this.minPool = new JTextField(20);
        this.maxPool = new JTextField(20);
        this.maxWaitTime = new JTextField(20);
        this.maxWaiters = new JTextField(20);
        this.pstmtPoolMax = new JTextField(20);
        this.sampling = new JTextField(20);
        this.adjustPeriod = new JTextField(20);
        this.test = new JTextField(20);
        this.dbName = new JTextField(20);
        this.dbPort = new JTextField(20);
        this.dbUser = new JTextField(20);
        this.dbUserPasswd = new JPasswordField(20);
        this.jmsPort = new JTextField(20);
        this.jmsTopics = new JTextField(20);
        this.jmsQueues = new JTextField(20);

        this.jbSelectAll = new JButton("Select all");
        this.jbDeselectAll = new JButton("Deselect all");
        this.jbBrowseJdbc = new JButton("Browse...");
        this.jbSaveDatasource = new JButton("Save/Add new");
        this.jbNewDatasource = new JButton("New Datasource");
        this.jbDeleteDatasource = new JButton("Delete");
        this.jbResetDatasource = new JButton("Reset");
        this.jbAdvancedDsConf = new JButton("Advanced config");
        this.jbSaveAdvancedDsConf = new JButton("Save");
        this.jbResetAdvancedDsConf = new JButton("Reset");
        this.jbAddJmsQueues = new JButton("Add");
        this.jbAddJmsTopics = new JButton("Add");
        this.jbRemoveJmsQueues = new JButton("Remove");
        this.jbRemoveJmsTopics = new JButton("Remove");

        this.jmsQueuesList = new JList();
        this.jmsTopicsList = new JList();

        this.checkingLevel = new JComboBox(new String[] {"0", "1", "2", "3"});

        this.useSsl = new JCheckBox("Use SSL");
        this.jvmRoute = new JCheckBox("Activate JVM route (optional)");
        // jkPort = new JCheckBox("Activate Jk port (optional)");

        this.jdbcDatasources = new ArrayList<String>();

        // applyConfiguration = new JCheckBox("Apply configuration", true);

        initDefaultValues();

        addWebContainerComponents();
        addProtocolComponents();
        addServicesComponents();
        addJmsComponents();
        addHsqlDbComponents();
        addDatasourceComponents();
        addAdvancedDatasourceComponents(-1);
        addTabbedPane();

        contentPanel1.setLayout(new BorderLayout());
        contentPanel1.add(this.tabbedPane, BorderLayout.NORTH);
        return contentPanel1;
    }

    /**
     * @return imageIcon. the image icon.
     */
    private ImageIcon getImageIcon() {
        return null;
    }

    /**
     * @return servicesCheckBox.
     */
    public ArrayList<JCheckBox> getServicesCheckBox() {
        return this.servicesCheckBox;
    }

    /**
     * @return jndiName
     */
    public JTextField getJndiName() {
        return this.jndiName;
    }

    /**
     * @return datasourceName
     */
    public JTextField getDatasourceName() {
        return this.datasourceName;
    }

    /**
     * @return carolLabel
     */
    public JLabel getCarolLabel() {
        return this.carolLabel;
    }

    /**
     * @return carolPort
     */
    public JTextField getCarolPort() {
        return this.carolPort;
    }

    /**
     * @return cbIiop
     */
    public JCheckBox getCbIiop() {
        return this.cbIiop;
    }

    /**
     * @return redirectPort
     */
    public JTextField getRedirectPort() {
        return this.redirectPort;
    }

    /**
     * @return implemClass.
     */
    public JTextField getImplemClass() {
        return this.implemClass;
    }

    /**
     * @return cbIrmi
     */
    public JCheckBox getCbIrmi() {
        return this.cbIrmi;
    }

    /**
     * @return cbJrmp
     */
    public JCheckBox getCbJrmp() {
        return this.cbJrmp;
    }

    /**
     * @return cbLmi
     */
    public JCheckBox getCbLmi() {
        return this.cbLmi;
    }

    /**
     * @return copyJdbc
     */
    public JRadioButton getCopyJdbc() {
        return this.copyJdbc;
    }

    /**
     * @return datasourceUrl
     */
    public JTextField getDatasourceUrl() {
        return this.datasourceUrl;
    }

    /**
     * @return datasourceClassName
     */
    public JTextField getDatasourceClassName() {
        return this.datasourceClassName;
    }

    /**
     * @return datasourceMapper
     */
    public JTextField getDatasourceMapper() {
        return this.datasourceMapper;
    }

    /**
     * @return dbms
     */
    public JComboBox getDbms() {
        return this.dbms;
    }

    /**
     * @return driverVersionComboBox
     */
    public JComboBox getDriverVersionComboBox() {
        return this.driverVersionComboBox;
    }

    /**
     * @return driverVersionTextField
     */
    public JTextField getDriverVersionTextField() {
        return this.driverVersionTextField;
    }

    /**
     * @return checkingLevel
     */
    public JComboBox getCheckingLevel() {
        return this.checkingLevel;
    }

    /**
     * @return downloadJdbcFromUrl
     */
    public JRadioButton getDownloadJdbcFromUrl() {
        return this.downloadJdbcFromUrl;
    }

    /**
     * @return downloadJdbcFromVersion
     */
    public JRadioButton getDownloadJdbcFromVersion() {
        return this.downloadJdbcFromVersion;
    }

    /**
     * @return hsqlPort
     */
    public JTextField getHsqlPort() {
        return this.hsqlPort;
    }

    /**
     * @return httpPort
     */
    public JTextField getHttpPort() {
        return this.httpPort;
    }

    /**
     * @return httpsPort
     */
    public JTextField getHttpsPort() {
        return this.httpsPort;
    }

    /**
     * @return iiop
     */
    public JTextField getIiop() {
        return this.iiop;
    }

    /**
     * @return irmi
     */
    public JTextField getIrmi() {
        return this.irmi;
    }

    /**
     * @return jbBrowseJdbc
     */
    public JButton getJbBrowseJdbc() {
        return this.jbBrowseJdbc;
    }

    /**
     * @return jbCancelConfig
     */
    public JButton getJbCancelConfig() {
        return this.jbCancelConfig;
    }

    /**
     * @return jbSelectAll
     */
    public JButton getJbSelectAll() {
        return this.jbSelectAll;
    }

    /**
     * @return jbDeselectAll
     */
    public JButton getJbDeselectAll() {
        return this.jbDeselectAll;
    }

    /**
     * @return jbValidateConfig
     */
    public JButton getJbValidateConfig() {
        return this.jbValidateConfig;
    }

    /**
     * @return jbSaveDatasource
     */
    public JButton getJbSaveDatasource() {
        return this.jbSaveDatasource;
    }

    /**
     * @return jbNewDatasource
     */
    public JButton getJbNewDatasource() {
        return this.jbNewDatasource;
    }

    /**
     * @return jbDeleteDatasource
     */
    public JButton getJbDeleteDatasource() {
        return this.jbDeleteDatasource;
    }

    /**
     * @return jbResetDatasource
     */
    public JButton getJbResetDatasource() {
        return this.jbResetDatasource;
    }

    /**
     * @return jbAdvancedDsConf
     */
    public JButton getJbAdvancedDsConf() {
        return this.jbAdvancedDsConf;
    }

    /**
     * @return jbSaveAdvancedDsConf
     */
    public JButton getJbSaveAdvancedDsConf() {
        return this.jbSaveAdvancedDsConf;
    }

    /**
     * @return jbResetAdvancedDsConf
     */
    public JButton getJbResetAdvancedDsConf() {
        return this.jbResetAdvancedDsConf;
    }

    /**
     * @return jdbcDriverPath
     */
    public JTextField getJdbcDriverPath() {
        return this.jdbcDriverPath;
    }

    /**
     * @return jdbcDriverUrl
     */
    public JTextField getJdbcDriverUrl() {
        return this.jdbcDriverUrl;
    }

    /**
     * @return jrmp
     */
    public JTextField getJrmp() {
        return this.jrmp;
    }

    /**
     * @return lmi
     */
    public JTextField getLmi() {
        return this.lmi;
    }

    /**
     * @return passwd
     */
    public JPasswordField getPasswd() {
        return this.passwd;
    }

    /**
     * @return tabbedPane
     */
    public JTabbedPane getTabbedPane() {
        return this.tabbedPane;
    }

    /**
     * @return user
     */
    public JTextField getUser() {
        return this.user;
    }

    /**
     * @return adjustPeriod
     */
    public JTextField getAdjustPeriod() {
        return this.adjustPeriod;
    }

    /**
     * @return maxAge
     */
    public JTextField getMaxAge() {
        return this.maxAge;
    }

    /**
     * @return maxOpenTime
     */
    public JTextField getMaxOpenTime() {
        return this.maxOpenTime;
    }

    /**
     * @return maxPool
     */
    public JTextField getMaxPool() {
        return this.maxPool;
    }

    /**
     * @return maxWaitTime
     */
    public JTextField getMaxWaitTime() {
        return this.maxWaitTime;
    }

    /**
     * @return maxWaiters
     */
    public JTextField getMaxWaiters() {
        return this.maxWaiters;
    }

    /**
     * @return minPool
     */
    public JTextField getMinPool() {
        return this.minPool;
    }

    /**
     * @return pstmtPoolMax
     */
    public JTextField getPstmtPoolMax() {
        return this.pstmtPoolMax;
    }

    /**
     * @return sampling
     */
    public JTextField getSampling() {
        return this.sampling;
    }

    /**
     * @return test
     */
    public JTextField getTest() {
        return this.test;
    }

    /**
     * @return jkPort
     */
    /*
     * public JCheckBox getJkPort() { return jkPort; }
     */

    /**
     * @return jvmRoute
     */
    public JCheckBox getJvmRoute() {
        return this.jvmRoute;
    }

    /**
     * @return useSsl
     */
    public JCheckBox getUseSsl() {
        return this.useSsl;
    }

    /**
     * @return jdbcDatasources
     */
    public List<String> getJdbcDatasources() {
        return this.jdbcDatasources;
    }

    /**
     * Set the datasource.
     * @param jdbcDatasources the datasource new value
     */
    public void setJdbcDatasources(final List<String> jdbcDatasources) {
        this.jdbcDatasources = jdbcDatasources;
    }

    /**
     * @return datasourceList
     */
    public JList getDatasourceList() {
        return this.datasourceList;
    }

    /**
     * @return advancedDatasourceList
     */
    public JList getAdvancedDatasourceList() {
        return this.advancedDatasourceList;
    }

    /**
     * @return datasourceItems
     */
    public List<String> getDatasourceItems() {
        return this.datasourceItems;
    }

    /**
     * Set the datasource item.
     * @param datasourceItems the datasource items new value
     */
    public void setDatasourceItems(final List<String> datasourceItems) {
        this.datasourceItems = datasourceItems;
    }

    /**
     * @return datasourceVersionItems
     */
    /*
     * public List<String> getDatasourceVersionItems() { return
     * datasourceVersionItems; }
     */

    /**
     * Set the datasourceVersion item.
     * @param datasourceVersionItems the datasourceVersion items new value
     */
    /*
     * public void setDatasourceVersionItems( final List<String>
     * datasourceVersionItems) { this.datasourceVersionItems =
     * datasourceVersionItems; }
     */

    /**
     * Set the datasource list.
     * @param datasourceList the datasource list new value
     */
    public void setDatasourceList(final JList datasourceList) {
        this.datasourceList = datasourceList;
    }

    /**
     * Set the advanced datasource list.
     * @param advancedDatasourceList the advancedDatasource list new value
     */
    public void setAdvancedDatasourceList(final JList advancedDatasourceList) {
        this.advancedDatasourceList = advancedDatasourceList;
    }

    /**
     * @return dbName
     */
    public JTextField getDbName() {
        return this.dbName;
    }

    /**
     * Set the dbName.
     * @param dbName new value
     */
    public void setDbName(final JTextField dbName) {
        this.dbName = dbName;
    }

    /**
     * @return dbPort
     */
    public JTextField getDbPort() {
        return this.dbPort;
    }

    /**
     * Set the dbPort.
     * @param dbPort new value
     */
    public void setDbPort(final JTextField dbPort) {
        this.dbPort = dbPort;
    }

    /**
     * @return dbUser
     */
    public JTextField getDbUser() {
        return this.dbUser;
    }

    /**
     * Set the dbUser.
     * @param dbUser new value
     */
    public void setDbUser(final JTextField dbUser) {
        this.dbUser = dbUser;
    }

    /**
     * @return dbUserPasswd
     */
    public JPasswordField getDbUserPasswd() {
        return this.dbUserPasswd;
    }

    /**
     * Set the dbUserPasswd.
     * @param dbUserPasswd new value
     */
    public void setDbUserPasswd(final JPasswordField dbUserPasswd) {
        this.dbUserPasswd = dbUserPasswd;
    }

    /**
     * @return the rarLink.
     */
    public JComboBox getRarLink() {
        return this.rarLink;
    }

    /**
     * @return the raName.
     */
    public JTextField getRaName() {
        return this.raName;
    }

    /**
     * @return the jmsPort.
     */
    public JTextField getJmsPort() {
        return this.jmsPort;
    }

    /**
     * @return jmsHostName
     */
    /*
     * public JTextField getJmsHostName() { return jmsHostName; }
     */

    /**
     * @return jmsServerId
     */
    /*
     * public JTextField getJmsServerId() { return jmsServerId; }
     */

    /**
     * @return the jmsQueues.
     */
    public JTextField getJmsQueues() {
        return this.jmsQueues;
    }

    /**
     * @return the jmsTopics.
     */
    public JTextField getJmsTopics() {
        return this.jmsTopics;
    }

    /**
     * @return jbAddJmsQueues.
     */
    public JButton getJbAddJmsQueues() {
        return this.jbAddJmsQueues;
    }

    /**
     * @return jbAddJmsTopics.
     */
    public JButton getJbAddJmsTopics() {
        return this.jbAddJmsTopics;
    }

    /**
     * @return jbRemoveJmsQueues.
     */
    public JButton getJbRemoveJmsQueues() {
        return this.jbRemoveJmsQueues;
    }

    /**
     * @return jbRemoveJmsTopics.
     */
    public JButton getJbRemoveJmsTopics() {
        return this.jbRemoveJmsTopics;
    }

    /**
     * @return jmsQueuesItems.
     */
    public List<String> getJmsQueuesItems() {
        return this.jmsQueuesItems;
    }

    /**
     * @return jmsQueuesList.
     */
    public JList getJmsQueuesList() {
        return this.jmsQueuesList;
    }

    /**
     * @return jmsTopicsItems.
     */
    public List<String> getJmsTopicsItems() {
        return this.jmsTopicsItems;
    }

    /**
     * @return jmsTopicsList.
     */
    public JList getJmsTopicsList() {
        return this.jmsTopicsList;
    }

    /**
     * @return driverName.
     */
    public JTextField getDriverName() {
        return this.driverName;
    }

    /**
     * @return applyConfiguration.
     */
    /*
     * public JCheckBox getApplyConfiguration() { return applyConfiguration; }
     */

    /**
     * initialize fields by the default values.
     */
    private void initDefaultValues() {
        this.httpPort.setText(this.configuration.getHttpPort());
        this.redirectPort.setText(this.configuration.getRedirectPort());
        this.useSsl.setSelected(this.configuration.getHttpsConnectorActivation());
        // jkPort.setSelected(configuration.getJkPortActivation());
        this.jvmRoute.setSelected(this.configuration.getJvmRouteActivation());

        /**
         * The JMS.
         */
        this.jmsPort.setText(this.configuration.getJms().getPort());
        this.jmsQueuesItems = this.configuration.getJms().getQueues();
        this.jmsTopicsItems = this.configuration.getJms().getTopics();

        /**
         * The HSQL DB.
         */
        this.dbName.setText(this.configuration.getDbName());
        this.dbPort.setText(this.configuration.getDbPort() + "");
        this.dbUser.setText(this.configuration.getDbUser());
        this.dbUserPasswd.setText(this.configuration.getDbUserPassword());

        /**
         * JDBC Configuration.
         */
        if (!this.configuration.getJdbcDatasource().isEmpty()) {
            JDBCDatasource jdbc = this.configuration.getJdbcDatasource().get(0);
            this.dbms.setSelectedItem(jdbc.getDbmsName());
            this.datasourceName.setText(jdbc.getDatasourceName());
            this.driverName.setText(jdbc.getJdbcConfiguration().driverName);
            this.jndiName.setText(jdbc.getJdbcConfiguration().jndiName);
            this.datasourceClassName.setText(jdbc.getJdbcConfiguration().datasourceClass);
            this.datasourceUrl.setText(jdbc.getJdbcConfiguration().url);
            this.datasourceMapper.setText(jdbc.getJdbcConfiguration().mappername);
            this.user.setText(jdbc.getJdbcConfiguration().user);
            this.passwd.setText(jdbc.getJdbcConfiguration().password);

            this.maxAge.setText(jdbc.getJdbcConfiguration().poolMaximumAgeMinutes);
            this.maxOpenTime.setText(jdbc.getJdbcConfiguration().poolMaximumOpenTime);
            this.checkingLevel.setSelectedItem(jdbc.getJdbcConfiguration().checkLevel);
            this.minPool.setText(jdbc.getJdbcConfiguration().poolMinimumSize);
            this.maxPool.setText(jdbc.getJdbcConfiguration().poolMaximumSize);
            this.maxWaitTime.setText(jdbc.getJdbcConfiguration().poolMaximumWaitTime);
            this.maxWaiters.setText(jdbc.getJdbcConfiguration().poolMaximumNumberOfWaiters);
            this.pstmtPoolMax.setText(jdbc.getJdbcConfiguration().poolMaximumNumberOfPreparedStatements);
            this.sampling.setText(jdbc.getJdbcConfiguration().poolSamplingPeriod);
            this.adjustPeriod.setText(jdbc.getJdbcConfiguration().poolInitialSize);
            this.test.setText(jdbc.getJdbcConfiguration().testStatement);
            this.rarLink.setSelectedItem(jdbc.getJdbcConfiguration().rarLink);
            this.raName.setText(jdbc.getRaName());
        }

        for (JDBCDatasource ds : this.configuration.getJdbcDatasource()) {
            this.jdbcDatasources.add(ds.getDatasourceName());
            // jdbcDatasources.add(ds.getJdbcConfiguration().jndiName);
        }
    }

    /**
     * add components to web container panel.
     */
    private void addWebContainerComponents() {
        JPanel webContainerPanelComp = new JPanel(new SpringLayout());
        /*
         * label = new JLabel("Web container", JLabel.TRAILING);
         * webContainerPanelComp.add(label); label.setLabelFor(tomcat); JPanel
         * plWebContener = new JPanel(new BorderLayout());
         * plWebContener.add(tomcat, BorderLayout.WEST);
         * plWebContener.add(jetty, BorderLayout.CENTER);
         * webContainerPanelComp.add(plWebContener);
         */

        this.label = new JLabel("HTTP port ", SwingConstants.TRAILING);
        webContainerPanelComp.add(this.label);
        this.label.setLabelFor(this.httpPort);
        webContainerPanelComp.add(this.httpPort);

        this.label = new JLabel("Redirect Port ", SwingConstants.TRAILING);
        webContainerPanelComp.add(this.label);
        this.label.setLabelFor(this.redirectPort);
        webContainerPanelComp.add(this.redirectPort);

        this.label = new JLabel("", SwingConstants.TRAILING);
        webContainerPanelComp.add(this.label);
        this.label.setLabelFor(this.useSsl);
        webContainerPanelComp.add(this.useSsl);

        this.label = new JLabel("", SwingConstants.TRAILING);
        webContainerPanelComp.add(this.label);
        this.label.setLabelFor(this.jvmRoute);
        webContainerPanelComp.add(this.jvmRoute);

        /*
         * label = new JLabel("", JLabel.TRAILING);
         * webContainerPanelComp.add(label); label.setLabelFor(jkPort);
         * webContainerPanelComp.add(jkPort);
         */

        SpringUtilities.makeCompactGrid(webContainerPanelComp, 4, 2, 6, 6, 6, 6);
        this.webContainerPanel.add(webContainerPanelComp, BorderLayout.NORTH);
        showSslOption(false);
    }

    /**
     * add components to protocol panel.
     */
    private void addProtocolComponents() {
        this.protocolPanelComponents = new JPanel(new SpringLayout());
        this.label = new JLabel("Carol protocols ", SwingConstants.TRAILING);
        this.protocolPanelComponents.add(this.label);
        this.label.setLabelFor(this.cbJrmp);
        JPanel plProto = new JPanel(new GridLayout());
        plProto.add(this.cbJrmp);
        plProto.add(this.cbIrmi);
        plProto.add(this.cbIiop);
        plProto.add(this.cbLmi);
        this.protocolPanelComponents.add(plProto);
        this.lines++;
        Protocol protocol = this.configuration.getProtocolByName("jrmp");
        if (this.cbJrmp.isSelected()) {
            this.jrmpLabel = new JLabel("JRMP Port ", SwingConstants.TRAILING);
            this.protocolPanelComponents.add(this.jrmpLabel);
            if (this.jrmp.getText().equals("")) {
                this.jrmp.setText("rmi://localhost:" + protocol.getPort());
            }
            this.jrmpLabel.setLabelFor(this.jrmp);
            this.protocolPanelComponents.add(this.jrmp);
            this.lines++;
        }

        protocol = this.configuration.getProtocolByName("irmi");
        if (this.cbIrmi.isSelected()) {
            this.irmiLabel = new JLabel("IRMI Port ", SwingConstants.TRAILING);
            this.protocolPanelComponents.add(this.irmiLabel);
            if (this.irmi.getText().equals("")) {
                this.irmi.setText("rmi://localhost:" + protocol.getPort());
            }
            this.irmiLabel.setLabelFor(this.irmi);
            this.protocolPanelComponents.add(this.irmi);
            this.lines++;
        }

        protocol = this.configuration.getProtocolByName("iiop");
        if (this.cbIiop.isSelected()) {
            this.iiopLabel = new JLabel("IIOP Port ", SwingConstants.TRAILING);
            this.protocolPanelComponents.add(this.iiopLabel);
            if (this.iiop.getText().equals("")) {
                this.iiop.setText("iiop://localhost:" + protocol.getPort());
            }
            this.iiopLabel.setLabelFor(this.iiop);
            this.protocolPanelComponents.add(this.iiop);
            this.lines++;
        }

        protocol = this.configuration.getProtocolByName("lmi");
        if (this.cbLmi.isSelected()) {
            this.lmiLabel = new JLabel("LMI Port ", SwingConstants.TRAILING);
            this.protocolPanelComponents.add(this.lmiLabel);
            if (this.lmi.getText().equals("")) {
                this.lmi.setText("rmi://localhost:" + protocol.getPort());
            }
            this.lmiLabel.setLabelFor(this.lmi);
            this.protocolPanelComponents.add(this.lmi);
            this.lines++;
        }

        SpringUtilities.makeCompactGrid(this.protocolPanelComponents, this.lines, this.columns, 6, 6, 6, 6);
        this.protocolPanel.add(this.protocolPanelComponents, BorderLayout.NORTH);
    }

    /**
     * Select protocols check box.
     */
    private void selectProtocolsCheckBox() {
        Protocol protocol = this.configuration.getProtocolByName("jrmp");
        boolean isEnabled = false;
        if (protocol != null) {
            isEnabled = protocol.getIsEnabled();
        }
        this.cbJrmp.setSelected(isEnabled);
        protocol = this.configuration.getProtocolByName("iiop");
        if (protocol != null) {
            isEnabled = protocol.getIsEnabled();
        }
        this.cbIiop.setSelected(isEnabled);
        protocol = this.configuration.getProtocolByName("irmi");
        if (protocol != null) {
            isEnabled = protocol.getIsEnabled();
        }
        this.cbIrmi.setSelected(isEnabled);
        protocol = this.configuration.getProtocolByName("lmi");
        if (protocol != null) {
            isEnabled = protocol.getIsEnabled();
        }
        this.cbLmi.setSelected(isEnabled);
    }

    /**
     * Show or hide JRMP fields.
     * @param visibility if true fields are showed, hided otherwise.
     */
    public void showJrmpfields(final boolean visibility) {
        if (visibility) {
            this.protocolPanel.remove(this.protocolPanelComponents);
            this.lines = 0;
            addProtocolComponents();
            this.jrmp.setFocusable(true);
            this.protocolPanel.revalidate();
        } else {
            this.protocolPanelComponents.remove(this.jrmpLabel);
            this.protocolPanelComponents.remove(this.jrmp);
            this.lines--;
            SpringUtilities.makeCompactGrid(this.protocolPanelComponents, this.lines, this.columns, 6, 6, 6, 6);
            this.protocolPanel.add(this.protocolPanelComponents, BorderLayout.NORTH);
            this.protocolPanel.revalidate();
        }
    }

    /**
     * Show or hide IRMI fields.
     * @param visibility if true fields are showed, hided otherwise.
     */
    public void showIrmifields(final boolean visibility) {
        if (visibility) {
            this.protocolPanel.remove(this.protocolPanelComponents);
            this.lines = 0;
            this.protocolPanel.revalidate();
            addProtocolComponents();
            this.protocolPanel.revalidate();
        } else {
            this.protocolPanelComponents.remove(this.irmiLabel);
            this.protocolPanelComponents.remove(this.irmi);
            this.lines--;
            SpringUtilities.makeCompactGrid(this.protocolPanelComponents, this.lines, this.columns, 6, 6, 6, 6);
            this.protocolPanel.add(this.protocolPanelComponents, BorderLayout.NORTH);
            this.protocolPanel.revalidate();
        }
    }

    /**
     * Show or hide IIOP fields.
     * @param visibility if true fields are showed, hided otherwise.
     */
    public void showIiopfields(final boolean visibility) {
        if (visibility) {
            this.protocolPanel.remove(this.protocolPanelComponents);
            this.lines = 0;
            this.protocolPanel.revalidate();
            addProtocolComponents();
            this.protocolPanel.revalidate();
        } else {
            this.protocolPanelComponents.remove(this.iiopLabel);
            this.protocolPanelComponents.remove(this.iiop);
            this.lines--;
            SpringUtilities.makeCompactGrid(this.protocolPanelComponents, this.lines, this.columns, 6, 6, 6, 6);
            this.protocolPanel.add(this.protocolPanelComponents, BorderLayout.NORTH);
            this.protocolPanel.revalidate();
        }
    }

    /**
     * Show or hide LMI fields.
     * @param visibility if true fields are showed, hided otherwise.
     */
    public void showLmifields(final boolean visibility) {
        if (visibility) {
            this.protocolPanel.remove(this.protocolPanelComponents);
            this.lines = 0;
            this.protocolPanel.revalidate();
            addProtocolComponents();
            this.protocolPanel.revalidate();
        } else {
            this.protocolPanelComponents.remove(this.lmiLabel);
            this.protocolPanelComponents.remove(this.lmi);
            this.lines--;
            SpringUtilities.makeCompactGrid(this.protocolPanelComponents, this.lines, this.columns, 6, 6, 6, 6);
            this.protocolPanel.add(this.protocolPanelComponents, BorderLayout.NORTH);
            this.protocolPanel.revalidate();
        }
    }

    /**
     * add components to services panel.
     */
    private void addServicesComponents() {
        JPanel servicesPanelComp = new JPanel(new SpringLayout());
        JPanel topPanel = new JPanel(new BorderLayout());
        JPanel middlePanel = new JPanel(new BorderLayout());
        JPanel bottomPanel = new JPanel(new BorderLayout());

        this.label = new JLabel("Select services to launch at" + " JOnAS server start up");
        topPanel.add(this.label, BorderLayout.NORTH);

        middlePanel.add(this.jbSelectAll, BorderLayout.WEST);
        middlePanel.add(this.jbDeselectAll, BorderLayout.EAST);

        List<String> srvName = new ArrayList<String>();
        int srvSize = this.configuration.getServices().size();
        for (int i = 0; i < srvSize; i++) {
            srvName.add(this.configuration.getServices().get(i).getName());
        }
        Collections.sort(srvName);
        JList servicesList = new JList(new Vector<String>(srvName));
        servicesList.setVisibleRowCount(4);
        this.servicesCheckBox = new ArrayList<JCheckBox>();
        JPanel p = new JPanel(new GridLayout(srvName.size(), 1));
        for (int i = 0; i < srvName.size(); i++) {
            JCheckBox serviceItem = new JCheckBox(srvName.get(i));
            if (this.configuration.getServiceByName(srvName.get(i)).getIsEnabled()) {
                serviceItem.setSelected(true);
            }
            p.add(serviceItem);
            this.servicesCheckBox.add(serviceItem);
        }
        JScrollPane servicesPane = new JScrollPane(p);
        servicesPane.setPreferredSize(new Dimension(500, 250));
        bottomPanel.add(servicesPane, BorderLayout.CENTER);

        servicesPanelComp.add(topPanel);
        servicesPanelComp.add(middlePanel);
        servicesPanelComp.add(bottomPanel);

        SpringUtilities.makeCompactGrid(servicesPanelComp, 3, 1, 6, 6, 6, 6);
        this.servicesPanel.add(servicesPanelComp, BorderLayout.NORTH);
    }

    /**
     * add components to jms panel.
     */
    private void addJmsComponents() {
        JPanel jmsPanelComp = new JPanel(new SpringLayout());

        JPanel jmsPanelJmsPort = new JPanel(new SpringLayout());
        this.label = new JLabel("JMS port ", SwingConstants.TRAILING);
        jmsPanelJmsPort.add(this.label);
        this.label.setLabelFor(this.jmsPort);
        jmsPanelJmsPort.add(this.jmsPort);
        SpringUtilities.makeCompactGrid(jmsPanelJmsPort, 1, 2, 6, 6, 6, 6);
        jmsPanelComp.add(jmsPanelJmsPort);

        JPanel jmsPanelQueues = new JPanel(new GridLayout(1, 2));
        JPanel jmsPanelQueuesContent = new JPanel(new SpringLayout());

        JPanel jmsPanelQueuesContentTextField = new JPanel(new SpringLayout());
        this.label = new JLabel("JMS queues ", SwingConstants.TRAILING);
        jmsPanelQueuesContentTextField.add(this.label);
        this.label.setLabelFor(this.jmsQueues);
        jmsPanelQueuesContentTextField.add(this.jmsQueues);
        SpringUtilities.makeCompactGrid(jmsPanelQueuesContentTextField, 1, 2, 6, 6, 6, 6);
        jmsPanelQueuesContent.add(jmsPanelQueuesContentTextField);

        JPanel jmsPanelQueuesContentButton = new JPanel();
        jmsPanelQueuesContentButton.add(this.jbAddJmsQueues);
        jmsPanelQueuesContentButton.add(this.jbRemoveJmsQueues);
        jmsPanelQueuesContent.add(jmsPanelQueuesContentButton);
        SpringUtilities.makeCompactGrid(jmsPanelQueuesContent, 2, 1, 6, 6, 6, 6);
        jmsPanelQueues.add(jmsPanelQueuesContent);
        JPanel panelJmsQueuesList = new JPanel(new BorderLayout());
        this.jmsQueuesList.setListData(new Vector<String>(this.jmsQueuesItems));
        panelJmsQueuesList.add(this.jmsQueuesList);
        JScrollPane jmsQueuesScrollPane = new JScrollPane(this.jmsQueuesList);
        jmsQueuesScrollPane.setPreferredSize(new Dimension(400, 10));
        panelJmsQueuesList.add(new JLabel("Queues list"), BorderLayout.NORTH);
        panelJmsQueuesList.add(jmsQueuesScrollPane, BorderLayout.CENTER);
        jmsPanelQueues.add(panelJmsQueuesList);

        JPanel jmsPanelTopics = new JPanel(new GridLayout(1, 2));
        JPanel jmsPanelTopicsContent = new JPanel(new SpringLayout());

        JPanel jmsPanelTopicsContentTextField = new JPanel(new SpringLayout());
        this.label = new JLabel("JMS topics ", SwingConstants.TRAILING);
        jmsPanelTopicsContentTextField.add(this.label);
        this.label.setLabelFor(this.jmsTopics);
        jmsPanelTopicsContentTextField.add(this.jmsTopics);
        SpringUtilities.makeCompactGrid(jmsPanelTopicsContentTextField, 1, 2, 6, 6, 6, 6);
        jmsPanelTopicsContent.add(jmsPanelTopicsContentTextField);

        JPanel jmsPanelTopicsContentButton = new JPanel();
        jmsPanelTopicsContentButton.add(this.jbAddJmsTopics);
        jmsPanelTopicsContentButton.add(this.jbRemoveJmsTopics);
        jmsPanelTopicsContent.add(jmsPanelTopicsContentButton);
        SpringUtilities.makeCompactGrid(jmsPanelTopicsContent, 2, 1, 6, 6, 6, 6);
        jmsPanelTopics.add(jmsPanelTopicsContent);
        JPanel panelJmsTopicsList = new JPanel(new BorderLayout());
        this.jmsTopicsList.setListData(new Vector<String>(this.jmsTopicsItems));
        panelJmsTopicsList.add(this.jmsTopicsList);
        JScrollPane jmsTopicsScrollPane = new JScrollPane(this.jmsTopicsList);
        jmsTopicsScrollPane.setPreferredSize(new Dimension(40, 10));
        panelJmsTopicsList.add(new JLabel("Topics list"), BorderLayout.NORTH);
        panelJmsTopicsList.add(jmsTopicsScrollPane, BorderLayout.CENTER);
        jmsPanelTopics.add(panelJmsTopicsList);

        jmsPanelComp.add(jmsPanelQueues);
        jmsPanelComp.add(jmsPanelTopics);

        SpringUtilities.makeCompactGrid(jmsPanelComp, 3, 1, 6, 6, 6, 6);
        this.jmsPanelComponents.add(jmsPanelComp, BorderLayout.NORTH);
    }

    /**
     * add components to hsqlDb panel.
     */
    private void addHsqlDbComponents() {
        JPanel hsqlPanelComp = new JPanel(new SpringLayout());

        this.label = new JLabel("DB name ", SwingConstants.TRAILING);
        hsqlPanelComp.add(this.label);
        this.label.setLabelFor(this.dbName);
        hsqlPanelComp.add(this.dbName);

        this.label = new JLabel("DB Port ", SwingConstants.TRAILING);
        hsqlPanelComp.add(this.label);
        this.label.setLabelFor(this.dbPort);
        hsqlPanelComp.add(this.dbPort);

        this.label = new JLabel("DB user ", SwingConstants.TRAILING);
        hsqlPanelComp.add(this.label);
        this.label.setLabelFor(this.dbUser);
        hsqlPanelComp.add(this.dbUser);

        this.label = new JLabel("DB user password", SwingConstants.TRAILING);
        hsqlPanelComp.add(this.label);
        this.label.setLabelFor(this.dbUserPasswd);
        hsqlPanelComp.add(this.dbUserPasswd);

        SpringUtilities.makeCompactGrid(hsqlPanelComp, 4, 2, 6, 6, 6, 6);
        this.hsqlPanelComponents.add(hsqlPanelComp, BorderLayout.NORTH);
    }

    /**
     * add components to datasource panel.
     */
    private void addDatasourceComponents() {
        JPanel datasourcePanelComp = new JPanel(new BorderLayout());
        JPanel datasourcePanelCompField = new JPanel(new BorderLayout());
        JPanel datasourcePanelCompList = new JPanel(new BorderLayout());
        JPanel datasourcePanelCompFieldItems = new JPanel(new BorderLayout());
        JPanel datasourcePanelItems = new JPanel(new SpringLayout());
        JPanel datasourcePanelButton = new JPanel();

        this.label = new JLabel("DBMS ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.dbms);
        datasourcePanelItems.add(this.dbms);

        /*
         * label = new JLabel("JDBC driver version ", JLabel.TRAILING);
         * datasourcePanelItems.add(label);
         * label.setLabelFor(driverVersionComboBox);
         */
        datasourcePanelItems.add(this.downloadJdbcFromVersion);
        JPanel driverVersion = new JPanel(new BorderLayout());
        driverVersion.add(this.driverVersionTextField, BorderLayout.WEST);
        driverVersion.add(this.driverVersionComboBox, BorderLayout.CENTER);
        datasourcePanelItems.add(driverVersion);

        datasourcePanelItems.add(this.downloadJdbcFromUrl);
        datasourcePanelItems.add(this.jdbcDriverUrl);
        enabledJdbcDriverFields();

        datasourcePanelItems.add(this.copyJdbc);
        JPanel pl = new JPanel(new BorderLayout());
        pl.add(this.jdbcDriverPath, BorderLayout.CENTER);
        pl.add(this.jbBrowseJdbc, BorderLayout.EAST);
        datasourcePanelItems.add(pl);

        this.label = new JLabel("Driver Name ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.driverName);
        datasourcePanelItems.add(this.driverName);
        this.label.setEnabled(false);
        this.driverName.setEnabled(false);

        this.label = new JLabel("RAR Name ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.raName);
        datasourcePanelItems.add(this.raName);
        this.label.setEnabled(true);

        this.label = new JLabel("RAR link ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.rarLink);
        datasourcePanelItems.add(this.rarLink);

        this.label = new JLabel("Datasource name ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.datasourceName);
        datasourcePanelItems.add(this.datasourceName);

        this.label = new JLabel("JNDI name ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.jndiName);
        datasourcePanelItems.add(this.jndiName);

        this.label = new JLabel("Datasource classname ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.datasourceClassName);
        datasourcePanelItems.add(this.datasourceClassName);

        this.label = new JLabel("Datasource URL ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.datasourceUrl);
        datasourcePanelItems.add(this.datasourceUrl);

        this.label = new JLabel("Datasource mapper ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.datasourceMapper);
        datasourcePanelItems.add(this.datasourceMapper);

        this.label = new JLabel("User name ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.user);
        datasourcePanelItems.add(this.user);

        this.label = new JLabel("User password ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.passwd);
        datasourcePanelItems.add(this.passwd);

        SpringUtilities.makeCompactGrid(datasourcePanelItems, 14, 2, 6, 6, 6, 6);
        datasourcePanelCompFieldItems.add(datasourcePanelItems, BorderLayout.NORTH);
        // datasourcePanelButton.add(jbNewDatasource);
        datasourcePanelButton.add(this.jbSaveDatasource);
        datasourcePanelButton.add(this.jbDeleteDatasource);
        datasourcePanelButton.add(this.jbResetDatasource);
        datasourcePanelButton.add(this.jbAdvancedDsConf);
        datasourcePanelCompFieldItems.add(datasourcePanelButton, BorderLayout.SOUTH);
        datasourcePanelCompField.add(datasourcePanelCompFieldItems, BorderLayout.SOUTH);
        this.datasourceList = new JList(new Vector<String>(this.jdbcDatasources));
        if (!this.jdbcDatasources.isEmpty()) {
            this.datasourceList.setSelectedValue(this.jdbcDatasources.get(0), true);
        }
        JScrollPane servicesPane = new JScrollPane(this.datasourceList);
        datasourcePanelCompList.add(new JLabel("List of datasources"), BorderLayout.NORTH);
        servicesPane.setPreferredSize(new Dimension(250, 0));
        datasourcePanelCompList.add(servicesPane, BorderLayout.CENTER);

        datasourcePanelComp.add(datasourcePanelCompField, BorderLayout.CENTER);
        datasourcePanelComp.add(datasourcePanelCompList, BorderLayout.EAST);
        this.datasourcePanel.add(datasourcePanelComp, BorderLayout.NORTH);
    }

    /**
     * add components to advanceddatasource panel.
     * @param index the selected datasource index.
     */
    public void addAdvancedDatasourceComponents(final int index) {
        JPanel datasourcePanelComp = new JPanel(new BorderLayout());
        JPanel datasourcePanelCompField = new JPanel(new BorderLayout());
        JPanel datasourcePanelCompList = new JPanel(new BorderLayout());
        JPanel datasourcePanelCompFieldItems = new JPanel(new BorderLayout());
        JPanel datasourcePanelItems = new JPanel(new SpringLayout());
        JPanel datasourcePanelButton = new JPanel();

        this.label = new JLabel("Maximum age ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.maxAge);
        datasourcePanelItems.add(this.maxAge);

        this.label = new JLabel("Maximum open time ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.maxOpenTime);
        datasourcePanelItems.add(this.maxOpenTime);

        this.label = new JLabel("Checking level ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.checkingLevel);
        datasourcePanelItems.add(this.checkingLevel);

        this.label = new JLabel("Pool minimum ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.minPool);
        datasourcePanelItems.add(this.minPool);

        this.label = new JLabel("Pool maximum ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.maxPool);
        datasourcePanelItems.add(this.maxPool);

        this.label = new JLabel("Max Wait Time ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.maxWaitTime);
        datasourcePanelItems.add(this.maxWaitTime);

        this.label = new JLabel("Max Waiters ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.maxWaiters);
        datasourcePanelItems.add(this.maxWaiters);

        this.label = new JLabel("Pstmt Pool Max ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.pstmtPoolMax);
        datasourcePanelItems.add(this.pstmtPoolMax);

        this.label = new JLabel("Sampling Period ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.sampling);
        datasourcePanelItems.add(this.sampling);

        this.label = new JLabel("Adjust Period ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.adjustPeriod);
        datasourcePanelItems.add(this.adjustPeriod);

        this.label = new JLabel("Test ", SwingConstants.TRAILING);
        datasourcePanelItems.add(this.label);
        this.label.setLabelFor(this.test);
        datasourcePanelItems.add(this.test);

        SpringUtilities.makeCompactGrid(datasourcePanelItems, 11, 2, 6, 6, 6, 6);
        datasourcePanelCompFieldItems.add(datasourcePanelItems, BorderLayout.NORTH);
        datasourcePanelButton.add(this.jbSaveAdvancedDsConf);
        datasourcePanelButton.add(this.jbResetAdvancedDsConf);
        datasourcePanelCompFieldItems.add(datasourcePanelButton, BorderLayout.SOUTH);
        datasourcePanelCompField.add(datasourcePanelCompFieldItems, BorderLayout.SOUTH);

        this.advancedDatasourceList = new JList(new Vector<String>(this.jdbcDatasources));
        if (index != -1) {
            this.advancedDatasourceList.setSelectedIndex(index);
        }
        JScrollPane servicesPane = new JScrollPane(this.advancedDatasourceList);
        datasourcePanelCompList.add(new JLabel("List of datasources"), BorderLayout.NORTH);
        servicesPane.setPreferredSize(new Dimension(250, 0));
        datasourcePanelCompList.add(servicesPane, BorderLayout.CENTER);

        datasourcePanelComp.add(datasourcePanelCompField, BorderLayout.CENTER);
        datasourcePanelComp.add(datasourcePanelCompList, BorderLayout.EAST);
        this.advancedConfigPanel.add(datasourcePanelComp, BorderLayout.NORTH);
    }

    /**
     * add tabbedPane to the frame.
     */
    private void addTabbedPane() {
        this.tabbedPane.addTab("Web container", this.webContainerPanel);
        this.tabbedPane.setMnemonicAt(0, KeyEvent.VK_W);
        this.tabbedPane.addTab("Carol protocols", this.protocolPanel);
        this.tabbedPane.setMnemonicAt(1, KeyEvent.VK_C);
        this.tabbedPane.addTab("Services", this.servicesPanel);
        this.tabbedPane.setMnemonicAt(2, KeyEvent.VK_S);
        this.tabbedPane.addTab("JMS", this.jmsPanelComponents);
        this.tabbedPane.setMnemonicAt(3, KeyEvent.VK_J);
        this.tabbedPane.addTab("HSQL DB", this.hsqlPanelComponents);
        this.tabbedPane.setMnemonicAt(4, KeyEvent.VK_H);
        this.tabbedPane.addTab("Datasource configuration", this.datasourcePanel);
        this.tabbedPane.setMnemonicAt(5, KeyEvent.VK_D);
        this.tabbedPane.addTab("Advanced configuration", this.advancedConfigPanel);
        this.tabbedPane.setMnemonicAt(6, KeyEvent.VK_A);
        this.tabbedPane.remove(this.advancedConfigPanel);
        this.mainPanel.add(this.tabbedPane);
    }

    /**
     * Select or deselect all services.
     * @param select if true, the services are seclected, deselected otherwise.
     */
    public void selectDeselectAllServices(final boolean select) {
        for (JCheckBox service : this.servicesCheckBox) {
            service.setSelected(select);
        }
    }

    /**
     * Show the SSL option fields.
     * @param enable if true, fields are showed, hided otherwise.
     */
    public void showSslOption(final boolean enable) {
        this.jvmRoute.setVisible(enable);
        // jkPort.setVisible(enable);
    }

    /**
     * Enable Jdbc Driver fields.
     */
    public void enabledJdbcDriverFields() {
        this.jdbcDriverPath.setEnabled(this.copyJdbc.isSelected());
        this.jbBrowseJdbc.setEnabled(this.copyJdbc.isSelected());
        this.jdbcDriverUrl.setEnabled(this.downloadJdbcFromUrl.isSelected());
        this.driverVersionTextField.setEnabled(this.downloadJdbcFromVersion.isSelected());
        this.driverVersionComboBox.setEnabled(this.downloadJdbcFromVersion.isSelected());
    }

    /**
     * Open Advanced Datasource Config tab.
     */
    public void openAdvancedDsConfTabbed() {
        this.tabbedPane.addTab("Advanced configuration", this.advancedConfigPanel);
        this.tabbedPane.setMnemonicAt(6, KeyEvent.VK_A);
        this.tabbedPane.setEnabledAt(6, true);
        this.tabbedPane.setSelectedIndex(6);
    }

    /**
     * Set the datasource fields values by the selected datasource.
     * @param datasource the selected datasource
     */
    public void setDatasourceFieldsValue(final String datasource) {
        int size = this.configuration.getJdbcDatasource().size();
        for (int i = 0; i < size; i++) {
            JDBCDatasource ds = this.configuration.getJdbcDatasource().get(i);
            if (ds.getDatasourceName().equals(datasource)) {
                this.datasourceName.setText(ds.getDatasourceName());
                this.dbms.setSelectedItem(ds.getDbmsName());
                this.driverName.setText(ds.getJdbcConfiguration().driverName);
                this.jndiName.setText(ds.getJdbcConfiguration().jndiName);
                this.datasourceClassName.setText(ds.getJdbcConfiguration().datasourceClass);
                this.datasourceUrl.setText(ds.getJdbcConfiguration().url);
                this.datasourceMapper.setText(ds.getJdbcConfiguration().mappername);
                this.user.setText(ds.getJdbcConfiguration().user);
                this.passwd.setText(ds.getJdbcConfiguration().password);
                this.raName.setText(ds.getRaName());
                this.rarLink.setSelectedItem(ds.getJdbcConfiguration().rarLink);
                return;
            }
        }
    }

    /**
     * Set the datasource advanced parameters the selected datasource.
     * @param datasource the selected datasource
     */
    public void setDatasourceAdvancedParametersValue(final String datasource) {
        int size = this.configuration.getJdbcDatasource().size();
        for (int i = 0; i < size; i++) {
            JDBCDatasource ds = this.configuration.getJdbcDatasource().get(i);
            if (ds.getDatasourceName().equals(datasource)) {
                this.maxAge.setText(ds.getJdbcConfiguration().poolMaximumAgeMinutes);
                this.maxOpenTime.setText(ds.getJdbcConfiguration().poolMaximumOpenTime);
                this.maxPool.setText(ds.getJdbcConfiguration().poolMaximumSize);
                this.maxWaitTime.setText(ds.getJdbcConfiguration().poolMaximumWaitTime);
                this.maxWaiters.setText(ds.getJdbcConfiguration().poolMaximumNumberOfWaiters);
                this.minPool.setText(ds.getJdbcConfiguration().poolMinimumSize);
                this.pstmtPoolMax.setText(ds.getJdbcConfiguration().poolMaximumNumberOfPreparedStatements);
                this.sampling.setText(ds.getJdbcConfiguration().poolSamplingPeriod);
                this.adjustPeriod.setText(ds.getJdbcConfiguration().poolInitialSize);
                this.test.setText(ds.getJdbcConfiguration().testStatement);
                this.checkingLevel.setSelectedItem(ds.getJdbcConfiguration().checkLevel);
                return;
            }
        }
    }

    /**
     * Return the default JNDI name of selected datasource.
     * @param datasource the selected datasource
     * @return datasourceDefaultJndiName.
     */
    public String getDatasourceDefaultJndiName(final String datasource) {
        int size = this.configuration.getJdbcDatasource().size();
        for (int i = 0; i < size; i++) {
            JDBCDatasource ds = this.configuration.getJdbcDatasource().get(i);
            if (ds.getJdbcConfiguration().jndiName.equals(datasource)) {
                return ds.getJdbcConfiguration().jndiName;
            }
        }
        return "";
    }

    /**
     * Return the default Url of selected datasource.
     * @param datasource the selected datasource
     * @return datasourceDefaultUrl.
     */
    public String getDatasourceDefaultUrl(final String datasource) {
        if (datasource.toLowerCase().equals("postgresql")) {
            return "jdbc:postgresql://<your-hostname>:5432/<db_name>";
        } else if (datasource.toLowerCase().equals("oracle")) {
            return "jdbc:oracle:thin:@<your-hostname>:1521:<your-db>";
        } else if (datasource.toLowerCase().equals("mysql")) {
            return "jdbc:mysql://<your-hostname>/3306/<your-db>";
        } else if (datasource.toLowerCase().equals("firebirdsql")) {
            return "jdbc:firebirdsql://<your-hostname>/<your-full-db-path>";
        } else if (datasource.toLowerCase().equals("interbase")) {
            return "jdbc:interbase://<your-hostname>/<your-full-db-path>";
        } else if (datasource.toLowerCase().equals("sybase")) {
            return "jdbc:sybase:Tds:<machineName>:5000/<your-db>";
        } else if (datasource.toLowerCase().equals("instantdb")) {
            return "jdbc:idb=db1.prp";
        } else if (datasource.toLowerCase().equals("mckoi")) {
            return "jdbc:mckoi://<your-hostname>";
        } else if (datasource.toLowerCase().equals("db2")) {
            return "jdbc:db2://<your.db.server>:51000/<your-db>";
        } else if (datasource.toLowerCase().equals("hsql")) {
            return "jdbc:hsqldb:hsql://localhost:9001/<your-db>";
        }
        return "";
    }

    /**
     * Return the default class name of selected datasource.
     * @param datasource the selected datasource
     * @return datasourceDefaultClassName.
     */
    public String getDatasourceDefaultClassName(final String datasource) {
        if (datasource.toLowerCase().equals("postgresql")) {
            return "org.postgresql.Driver";
        } else if (datasource.toLowerCase().equals("oracle")) {
            return "oracle.jdbc.driver.OracleDriver";
        } else if (datasource.toLowerCase().equals("mysql")) {
            return "org.gjt.mm.mysql.Driver";
        } else if (datasource.toLowerCase().equals("firebirdsql")) {
            return "org.firebirdsql.jdbc.FBDriver";
        } else if (datasource.toLowerCase().equals("interbase")) {
            return "interbase.interclient.Driver";
        } else if (datasource.toLowerCase().equals("sybase")) {
            return "com.sybase.jdbc2.jdbc.SybDriver";
        } else if (datasource.toLowerCase().equals("instantdb")) {
            return "org.enhydra.instantdb.jdbc.idbDriver";
        } else if (datasource.toLowerCase().equals("mckoi")) {
            return "com.mckoi.JDBCDriver";
        } else if (datasource.toLowerCase().equals("db2")) {
            return "COM.ibm.db2.jdbc.net.DB2Driver";
        } else if (datasource.toLowerCase().equals("hsql")) {
            return "org.hsqldb.jdbcDriver";
        }
        return "";
    }

    /**
     * Return the default mapper of selected datasource.
     * @param datasource the selected datasource
     * @return datasourceDefaultMapper.
     */
    public String getDatasourceDefaultMapper(final String datasource) {
        if (datasource.toLowerCase().equals("postgresql")) {
            return "rdb.postgres";
        } else if (datasource.toLowerCase().equals("oracle")) {
            return "rdb.oracle";
        } else if (datasource.toLowerCase().equals("mysql")) {
            return "rdb.mysql";
        } else if (datasource.toLowerCase().equals("firebirdsql")) {
            return "rdb.firebird";
        } else if (datasource.toLowerCase().equals("interbase")) {
            return "rdb.interbase";
        } else if (datasource.toLowerCase().equals("sybase")) {
            return "";
        } else if (datasource.toLowerCase().equals("instantdb")) {
            return "";
        } else if (datasource.toLowerCase().equals("mckoi")) {
            return "rdb.mckoi";
        } else if (datasource.toLowerCase().equals("db2")) {
            return "rdb.db2";
        } else if (datasource.toLowerCase().equals("hsql")) {
            return "rdb.hsql";
        }
        return "";
    }

    /**
     * Reset the datasource fields.
     */
    public void resetDatasourceField() {
        this.jdbcDriverPath.setText("");
        this.jdbcDriverUrl.setText("");
        this.copyJdbc.setSelected(true);
        this.downloadJdbcFromUrl.setSelected(false);
        this.downloadJdbcFromVersion.setSelected(false);
        this.driverName.setText("");
        this.datasourceName.setText("");
        this.jndiName.setText("");
        this.datasourceClassName.setText("");
        this.datasourceMapper.setText("");
        this.datasourceUrl.setText("");
        this.raName.setText("");
        this.rarLink.setSelectedIndex(-1);
        this.user.setText("");
        this.passwd.setText("");
        this.datasourceList.setSelectedIndex(-1);
        this.advancedDatasourceList.setSelectedIndex(-1);
        this.dbms.setSelectedIndex(-1);
        this.driverVersionComboBox.setSelectedIndex(-1);
        this.driverVersionTextField.setText("");
    }

    /**
     * Reset the advancedConfig fields.
     */
    public void resetAdvancedConfigField() {
        this.checkingLevel.setSelectedIndex(0);
        this.maxAge.setText("");
        this.maxOpenTime.setText("");
        this.maxPool.setText("");
        this.maxWaitTime.setText("");
        this.maxWaiters.setText("");
        this.pstmtPoolMax.setText("");
        this.minPool.setText("");
        this.sampling.setText("");
        this.adjustPeriod.setText("");
        this.test.setText("");
    }

    /**
     * Save datasource.
     * @param dsName the selected datasource name. Null value means it is a new
     *        datasource to add, otherwise it is an update.
     */
    public void updateBuilderDatasources(final String dsName) {
        if (dsName == null) {
            // Add a new datasource
            JDBCDatasource newDatasource = new JDBCDatasource();
            setJdbcDatasourceValue(newDatasource);
            this.configuration.addJdbcDatasource(newDatasource);
            // jdbcDatasources.add(newDatasource.getJdbcConfiguration().jndiName);
            this.jdbcDatasources.add(newDatasource.getDatasourceName());
            Collections.sort(this.jdbcDatasources);
            this.datasourceList.setListData(new Vector<String>(this.jdbcDatasources));
            this.advancedDatasourceList.setListData(new Vector<String>(this.jdbcDatasources));
            // resetDatasourceField();
            this.datasourceList.setSelectedIndex(-1);
        } else {
            int size = this.configuration.getJdbcDatasource().size();
            for (int i = 0; i < size; i++) {
                JDBCDatasource ds = this.configuration.getJdbcDatasource().get(i);
                if (ds.getDatasourceName().equals(dsName)) {
                    // if (ds.getJdbcConfiguration().jndiName.equals(dsName)) {
                    setJdbcDatasourceValue(ds);
                    this.configuration.getJdbcDatasource().set(i, ds);
                    this.jdbcDatasources.set(i, ds.getDatasourceName());
                    Collections.sort(this.jdbcDatasources);
                    this.datasourceList.setListData(new Vector<String>(this.jdbcDatasources));
                    this.advancedDatasourceList.setListData(new Vector<String>(this.jdbcDatasources));
                    // resetDatasourceField();
                    this.datasourceList.setSelectedIndex(-1);
                    return;
                }
            }
        }
    }

    /**
     * Set the jdbc datasource value.
     * @param currentDatasource the selected datasource
     */
    private void setJdbcDatasourceValue(final JDBCDatasource currentDatasource) {
        currentDatasource.setDatasourceName(this.datasourceName.getText());
        currentDatasource.setRaName(this.raName.getText());
        currentDatasource.setDbmsName((String) this.dbms.getSelectedItem());
        currentDatasource.getJdbcConfiguration().driverName = System.getProperty("dbJdbcDriverPath");
        currentDatasource.getJdbcConfiguration().jndiName = this.jndiName.getText();
        currentDatasource.getJdbcConfiguration().datasourceClass = this.datasourceClassName.getText();
        currentDatasource.getJdbcConfiguration().url = this.datasourceUrl.getText();
        currentDatasource.getJdbcConfiguration().mappername = this.datasourceMapper.getText();
        currentDatasource.getJdbcConfiguration().user = this.user.getText();
        currentDatasource.getJdbcConfiguration().password = new String(this.passwd.getPassword());
        currentDatasource.getJdbcConfiguration().rarLink = (String) this.rarLink.getSelectedItem();
        currentDatasource.initializeJdbcConfigurationAdvancedParameters();
    }

    /**
     * Save advanced datasource config.
     * @param dsName the selected datasource name
     */
    public void saveAdvancedDsConfig(final String dsName) {
        setDatasourceAdvancedValue(dsName);
        this.tabbedPane.remove(this.advancedConfigPanel);
        this.tabbedPane.setSelectedIndex(5);
    }

    /**
     * Set the datasource advanced paramters.
     * @param dsName the selected datasource name
     */
    private void setDatasourceAdvancedValue(final String dsName) {
        int size = this.configuration.getJdbcDatasource().size();
        for (int i = 0; i < size; i++) {
            JDBCDatasource ds = this.configuration.getJdbcDatasource().get(i);
            if (ds.getDatasourceName().equals(dsName)) {
                // if (ds.getJdbcConfiguration().jndiName.equals(dsName)) {
                ds.getJdbcConfiguration().poolMaximumAgeMinutes = this.maxAge.getText();
                ds.getJdbcConfiguration().poolMaximumOpenTime = this.maxOpenTime.getText();
                ds.getJdbcConfiguration().poolMaximumSize = this.maxPool.getText();
                ds.getJdbcConfiguration().poolMaximumWaitTime = this.maxWaitTime.getText();
                ds.getJdbcConfiguration().poolMaximumNumberOfWaiters = this.maxWaiters.getText();
                ds.getJdbcConfiguration().poolMinimumSize = this.minPool.getText();
                ds.getJdbcConfiguration().poolMaximumNumberOfPreparedStatements = this.pstmtPoolMax.getText();
                ds.getJdbcConfiguration().poolSamplingPeriod = this.sampling.getText();
                ds.getJdbcConfiguration().poolInitialSize = this.adjustPeriod.getText();
                ds.getJdbcConfiguration().testStatement = this.test.getText();
                ds.getJdbcConfiguration().checkLevel = (String) this.checkingLevel.getSelectedItem();
                return;
            }
        }
    }

    /**
     * Delete datasource.
     * @param dsName the selected datasource name.
     */
    public void deleteDatasource(final String dsName) {
        int size = this.configuration.getJdbcDatasource().size();
        for (int i = 0; i < size; i++) {
            JDBCDatasource ds = this.configuration.getJdbcDatasource().get(i);
            if (ds.getDatasourceName().equals(dsName)) {
                // if (ds.getJdbcConfiguration().jndiName.equals(dsName)) {
                this.configuration.removeJdbcDatasource(ds);
                this.jdbcDatasources.remove(dsName);
                Collections.sort(this.jdbcDatasources);
                this.datasourceList.setListData(new Vector<String>(this.jdbcDatasources));
                this.advancedDatasourceList.setListData(new Vector<String>(this.jdbcDatasources));
                resetDatasourceField();
                this.datasourceList.setSelectedIndex(-1);
                return;
            }
        }
    }

    /**
     * Update the builder services.
     */
    public void updateBuilderServices() {
        Vector<Service> services = new Vector<Service>();
        for (JCheckBox srvCheckBox : this.servicesCheckBox) {
            services.add(new Service(srvCheckBox.getText(), srvCheckBox.isSelected()));
        }
        this.configuration.setServices(services);
    }

    /**
     * Update the builder protocols.
     */
    public void updateBuilderProtocols() {
        if (this.cbJrmp.isSelected()) {
            String portStr = this.jrmp.getText().substring(this.jrmp.getText().lastIndexOf(":") + 1,
                    this.jrmp.getText().length());
            setProtocolValue(this.cbJrmp.getText().toLowerCase(), portStr, this.cbJrmp.isSelected());
        }

        if (this.cbIiop.isSelected()) {
            String portStr = this.iiop.getText().substring(this.iiop.getText().lastIndexOf(":") + 1,
                    this.iiop.getText().length());
            setProtocolValue(this.cbIiop.getText().toLowerCase(), portStr, this.cbIiop.isSelected());
        }

        if (this.cbIrmi.isSelected()) {
            String portStr = this.irmi.getText().substring(this.irmi.getText().lastIndexOf(":") + 1,
                    this.irmi.getText().length());
            setProtocolValue(this.cbIrmi.getText().toLowerCase(), portStr, this.cbIrmi.isSelected());
        }

        if (this.cbLmi.isSelected()) {
            String portStr = this.lmi.getText().substring(this.lmi.getText().lastIndexOf(":") + 1, this.lmi.getText().length());
            setProtocolValue(this.cbLmi.getText().toLowerCase(), portStr, this.cbLmi.isSelected());
        }
    }

    /**
     * Set the protocol value.
     * @param protocolName the name of protocol.
     * @param port the port value.
     * @param isSelected assert if the protocol is selected
     */
    private void setProtocolValue(final String protocolName, final String port, final boolean isSelected) {
        if (this.configuration.getProtocolByName(protocolName) != null) {
            this.configuration.getProtocolByName(protocolName).setName(protocolName);
            this.configuration.getProtocolByName(protocolName).setPort(port);
            this.configuration.getProtocolByName(protocolName).setIsEnabled(isSelected);
        }
    }

    /**
     * Update the builder JMS values.
     */
    public void updateBuilderJms() {
        this.configuration.getJms().setPort(this.jmsPort.getText());
        this.configuration.getJms().setQueues(this.jmsQueuesItems);
        this.configuration.getJms().setTopics(this.jmsTopicsItems);
    }

    /**
     * Update the builder HSQL DB values.
     */
    public void updateBuilderHsqlDb() {
        this.configuration.setDbName(this.dbName.getText());
        this.configuration.setDbPort(this.dbPort.getText());
        this.configuration.setDbUser(this.dbUser.getText());
        this.configuration.setDbUserPassword(new String(this.dbUserPasswd.getPassword()));
    }

    /**
     * Update the builder web container values.
     */
    public void updateBuilderWebContainer() {
        this.configuration.setHttpPort(this.httpPort.getText());
        this.configuration.setRedirectPort(this.redirectPort.getText());
        this.configuration.setHttpsConnectorActivation(this.useSsl.isSelected());
        this.configuration.setJvmRouteActivation(this.jvmRoute.isSelected());
        /*
         * configuration.setJkPortActivation( jkPort.isSelected());
         */
    }

    /**
     * ActionListener function.
     * @param listener the listener.
     */
    public void addActionListener(final ActionListener listener) {
        this.useSsl.addActionListener(listener);
        this.cbJrmp.addActionListener(listener);
        this.cbIiop.addActionListener(listener);
        this.cbIrmi.addActionListener(listener);
        this.cbLmi.addActionListener(listener);

        this.jbSelectAll.addActionListener(listener);
        this.jbDeselectAll.addActionListener(listener);
        this.jbAdvancedDsConf.addActionListener(listener);
        this.jbDeleteDatasource.addActionListener(listener);
        this.jbNewDatasource.addActionListener(listener);
        this.jbResetDatasource.addActionListener(listener);
        this.jbSaveDatasource.addActionListener(listener);
        this.jbSaveAdvancedDsConf.addActionListener(listener);
        this.jbResetAdvancedDsConf.addActionListener(listener);
        this.jbBrowseJdbc.addActionListener(listener);
        this.jbAddJmsQueues.addActionListener(listener);
        this.jbAddJmsTopics.addActionListener(listener);
        this.jbRemoveJmsQueues.addActionListener(listener);
        this.jbRemoveJmsTopics.addActionListener(listener);

        this.copyJdbc.addActionListener(listener);
        this.downloadJdbcFromUrl.addActionListener(listener);
        this.downloadJdbcFromVersion.addActionListener(listener);
        this.dbms.addActionListener(listener);
        this.driverVersionComboBox.addActionListener(listener);
        // applyConfiguration.addActionListener(listener);
    }

    /**
     * ListSelectionListener function.
     * @param listener the listener.
     */
    public void addListSelectionListener(final ListSelectionListener listener) {
        this.datasourceList.addListSelectionListener(listener);
        this.advancedDatasourceList.addListSelectionListener(listener);
        this.jmsQueuesList.addListSelectionListener(listener);
        this.jmsTopicsList.addListSelectionListener(listener);
    }
}
